/**************************************************************
	
	cardclasses.cpp: objects and functions for cards and hands
	Copyright 2002, Jacob A. Ewing

	Author: Jacob A. Ewing
	e-mail:	jacob@diagrami.com

**************************************************************

    This file is part of Netcrib.

    Netcrib is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Netcrib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Netcrib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

**************************************************************/
#ifndef _STDLIB_H
#	include <stdlib.h>
#endif

#ifndef __NCURSES_H
#	include <ncurses.h>
#endif

#ifndef __CARDCLASSES_H
#	include "cardclasses.h"
#endif

void cardtype::displaysuit(void){
      switch(suit){
	      case Diamonds: printw("D"); break;
	      case Clubs: printw("C"); break;
	      case Hearts: printw("H"); break;
	      case Spades: printw("S"); break;
	      default: printw("E");
      }
}

void cardtype::displayvalue(void){
      if(value < 11 && value > 1)
	      printw("%i", value);
      else{
	      switch(value){
		      case 1: printw("A"); break;
		      case 11: printw("J"); break;
		      case 12: printw("Q"); break;
		      case 13: printw("K"); break;
		      default: printw("E");
	      }
      }
}

cardtype::cardtype(){ erase(); }

void cardtype::erase(void){
	suit = nosuit;
	value = '0';
	pointvalue = 0;
}


void cardtype::display(displaymodetype displaymode){
	int x, y, colournum;
	switch(displaymode){
	    case ansi:
		getyx(stdscr, y, x);
		if(suit == Hearts || suit == Diamonds)
			colournum = 2;
		else
			colournum = 1;

		move(y, x);
		attron(COLOR_PAIR(colournum));
		displayvalue();
		printw(" ");
		if(value != 10) printw(" ");

		move(y + 1, x);
		printw(" ");
		displaysuit();
		printw(" ");

		move(y + 2, x);
		printw(" ");
		if(value != 10) printw(" ");
		displayvalue();

		attroff(COLOR_PAIR(colournum));
	}
}

void cardtype::display(int y, int x){
	int colournum;
	if(suit == Hearts || suit == Diamonds)
		colournum = 2;
	else
		colournum = 1;

	move(y, x);
	attron(COLOR_PAIR(colournum));
	displayvalue();
	printw(" ");
	if(value != 10) printw(" ");

	move(0, 0);
	move(y + 1, x);
	printw(" ");
	displaysuit();
	printw(" ");

	move(0, 0);
	move(y + 2, x);
	printw(" ");
	if(value != 10) printw(" ");
	displayvalue();

	attroff(COLOR_PAIR(colournum));
}

cardtype cardtype::operator = (cardtype newcard){
	suit = newcard.suit;
	value = newcard.value;
	pointvalue = newcard.pointvalue;
	return newcard;
}

int cardtype::operator != (cardtype testcard){
	int returnval = 0;
	if(suit != testcard.suit) returnval = 1;
	if(value != testcard.value) returnval = 1;
	return returnval;
}

int cardtype::operator == (cardtype testcard){
	int returnval = 1;
	if(suit != testcard.suit) returnval = 0;
	if(value != testcard.value) returnval = 0;
	return returnval;
}


handtype::handtype(){
	/*** start with an empty hand ***/
	numcards = 0;
}

void handtype::erase(void){
	int n;
	for(n = 0; n < MAXHANDSIZE; n++){
		card[n].erase();
	}
	numcards = 0;
}

void handtype::display(displaymodetype displaymode){
	int n;
	for(n = 0; n < numcards; n++){
		card[n].display(displaymode);
	}
}

void handtype::display(int y, int x){
	int n;
	for(n = 0; n < numcards; n++){
		card[n].display(y, x + (n << 2));
	}
}


int handtype::sum(void){
	int n, returnval = 0;
	for(n = 0; n < numcards; n++){
		returnval += card[n].pointvalue;
	}

	return returnval;
}

handtype handtype::operator = (handtype newhand){
	int n;
	erase();
	numcards = newhand.numcards;
	for(n = 0; n < numcards; n++){
		card[n] = newhand.card[n];
	}
	return newhand;
}

cardtype handtype::operator += (cardtype newcard){
	if(numcards < MAXHANDSIZE){
		card[numcards] = newcard;
		numcards++;
	}
	return newcard;
}

void handtype::sort(void){
	cardtype tempcard;
	int n, m;
	for(n = 1; n < numcards; n++){
		m = n;
		/*** Move the value down to the appropriate spot. ***/
		while(m > 0 && card[m].value < card[m - 1].value){
			/*** switch the values around ***/
			tempcard = card[m];
			card[m] = card[m - 1];
			card[m - 1] = tempcard;
			m--;
		}
	}
}

void handtype::passcard(handtype *recipient, int cardindex){
	*recipient += card[cardindex];
	removecard(cardindex);
}

void handtype::removecard(int cardindex){
	int n;
	numcards--;
	for(n = cardindex; n < numcards; n++){
		card[n] = card[n + 1];
	}
	card[n].erase();
}


decktype::decktype(){
	int n;

	/*** first, create the cards ***/
	numcards = decksize;
	for(n = 0; n < decksize; n++){
		card[n].suit = (suittype)(n % 4);
		card[n].value = (n >> 2) + 1;
		card[n].pointvalue = card[n].value;
		if(card[n].pointvalue > 10) card[n].pointvalue = 10;
	}

	/*** now shuffle them ***/
	shuffle();
}

void decktype::shuffle(void){
	int n, m, numruns;
	cardtype swapcard;

	for(numruns = 0; numruns < 6; numruns++){
		for(n = 0; n < decksize; n++){

			if(rand() % 2){
				swapcard = card[n];
				for(m = n; m > 0; m--)
					card[m] = card[m - 1];
				card[0] = swapcard;
			}
		}
	}
}

void decktype::display(displaymodetype displaymode){
	int n;
	for(n = 0; n < numcards; n++){
		card[n].display(displaymode);
		if(n % 4 == 3) printw("\n");
	}
}

cardtype decktype::draw(int cutpos){
	cardtype returnval;
	int n;
	if(cutpos >= 0 && cutpos < numcards){
		returnval = card[cutpos];
		numcards--;
		for(n = cutpos; n < numcards; n++){
			card[n] = card[n + 1];
		}
		card[n].erase();
	}
	return returnval;
}

cardtype decktype::operator += (cardtype addition){
	if(numcards < decksize){
		card[numcards] = addition;
		numcards ++;
	}
	return addition;
}

handtype decktype::operator += (handtype addition){
	int n;
	if(numcards + addition.numcards <= decksize){
		for(n = 0; n < addition.numcards; n++){
			card[numcards] = addition.card[n];
			numcards ++;
		}
	}
	return addition;
}
